using System;
//using System.Collections.Generic;
using System.Collections;
using System.IO;
using System.Net;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Reflection;

using MapApp.Model;

namespace MapApp.UI
{
    public partial class FrmFind : AppFormBase
    {
        static string addressHistoryFileName = "addressHistory.txt";
        //static string historyFilePath = AppUtil.tempDir + "/" + addressHistoryFileName;
        //static string historyFilePath = "http://interchart.com/" + addressHistoryFileName;
        static string historyFilePath = "ftp://ftp.interchart.com/" + addressHistoryFileName;
        static string ftpUserName = "interchart.com";
        static string ftpPassword = "rkregister4less";
        static NetworkCredential cred = new NetworkCredential(ftpUserName, ftpPassword);

        Thread historyThread ; //= new Thread(new ThreadStart(HistoryWorker));

        public FrmFind()
        {
            InitializeComponent();
            string findFilePath = AppUtil.tempDir + "/find.html";
            AppUtil.writeResourceToDisk("MapApp.Resources.html.find.html", findFilePath);
            wbFind.Url = new Uri(findFilePath);

            cmbFind.AutoCompleteCustomSource = new AutoCompleteStringCollection();

            historyThread = new Thread(new ThreadStart(HistoryWorker));
            historyThread.Start();

            //LoadHistoryFile();
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            this.Hide();
        }

        private void wbFind_DocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
        {
            wbFind.Document.GetElementById("commsDiv").Click += new HtmlElementEventHandler(link_Click);
        }

        private void link_Click(object sender, HtmlElementEventArgs e)
        {
            string[] parts = wbFind.Document.GetElementById("commsDiv").InnerHtml.Split(':');
            string addr = wbFind.Document.GetElementById("addrDiv").InnerText;
            MapLocation zoomToLoc = new MapLocation(addr, Double.Parse(parts[0]), Double.Parse(parts[1]),
                    Int32.Parse(parts[2]));
            if (!cmbFind.AutoCompleteCustomSource.Contains(zoomToLoc.LocationString))
            {
                cmbFind.AutoCompleteCustomSource.Add(zoomToLoc.LocationString);
            }

            Map.NewMarker(zoomToLoc, addr);
            Map.ZoomTo(zoomToLoc);
        }

        private void cmbFind_KeyUp(object sender, KeyEventArgs e)
        {
            // If the ENTER key is pressed, the Handled property is set to true, 
            // to indicate the event is handled.
            if (e.KeyCode == Keys.Enter)
            {
                doSearch();
                // Tell system that the event was handled
                e.Handled = true;
            }

        }

        private void doSearch()
        {
            // Do the search:
            string searchText = cmbFind.Text;
            if (chkAustralia.Checked) searchText += " Australia";
            if (chkUSA.Checked) searchText += " USA";
            wbFind.Document.GetElementById("search").InnerText = searchText;
            Object obj = wbFind.Document.GetElementById("enter").DomElement;
            Type type = obj.GetType();
            MethodInfo[] methods = obj.GetType().GetMethods();
            MethodInfo mi = obj.GetType().GetMethod("click");
            mi.Invoke(obj, new object[0]);
        }

        private void chk_CheckedChanged(object sender, EventArgs e)
        {
            if ((sender == chkAustralia) && chkAustralia.Checked)
            {
                chkUSA.Checked = false;
            }
            if ((sender == chkUSA) && chkUSA.Checked)
            {
                chkAustralia.Checked = false;
            }
        }

        private void btnFind_Click(object sender, EventArgs e)
        {
            doSearch();
        }


        //////////////////////////////////////////////// History functions ///////////////////

        // Executes in historyThread
        void HistoryWorker() {
            LoadHistoryFile();
        }

        /// <summary>
        /// Serialize the location history to disk when the form closes
        /// Handle local disk files, http: and ftp: protocols
        /// </summary>
        public void SaveHistoryFile()
        {
          try {
            //return;
            if (cmbFind.AutoCompleteCustomSource != null) {
              StreamWriter sw = null;
              Boolean useReq = false;
              IEnumerator en = cmbFind.AutoCompleteCustomSource.GetEnumerator();
              WebRequest req = WebRequest.Create(historyFilePath);
              // sad, have to set things in a protocol dependent way...
              switch (req.GetType().Name) {
                case "FileWebRequest":
                  //FileWebRequest fileReq = (FileWebRequest)req;
                  sw = new StreamWriter(historyFilePath);  // Don't use the req object at all.....
                  useReq = false;
                  break;
                case "HttpWebRequest":
                  HttpWebRequest httpReq = (HttpWebRequest)req;
                  httpReq.Method = "PUT";
                  sw = new StreamWriter(req.GetRequestStream());
                  useReq = true;
                  break;
                case "FtpWebRequest":
                  FtpWebRequest ftpReq = (FtpWebRequest)req;
                  ftpReq.Method = WebRequestMethods.Ftp.UploadFile;
                  ftpReq.UseBinary = false; // We're getting a text file
                  ftpReq.KeepAlive = false; // Don't keep control channel open indefinitely
                  ftpReq.Credentials = new NetworkCredential(ftpUserName, ftpPassword);
                  sw = new StreamWriter(req.GetRequestStream());
                  useReq = false;
                  break;
              }
              ////if (req.GetType().FullName.Equals("System.Net.FileWebRequest"))
              ////{
              ////    //FileWebRequest fileReq = (FileWebRequest)req;
              ////    sw = new StreamWriter(historyFilePath);  // Don't use the req object at all.....
              ////    useReq = false;
              ////}
              ////if (req.GetType().FullName.Equals("System.Net.HttpWebRequest"))
              ////{
              ////    HttpWebRequest httpReq = (HttpWebRequest)req;
              ////    httpReq.Method = "PUT";
              ////    sw = new StreamWriter(req.GetRequestStream());
              ////    useReq = true;
              ////}
              ////if (req.GetType().FullName.Equals("System.Net.FtpWebRequest"))
              ////{
              ////    // ??????
              ////}
              //using (StreamWriter sw = new StreamWriter(historyFilePath))
              using (sw) {
                while (en.MoveNext()) {
                  string nextAddr = (String)en.Current;
                  sw.WriteLine(nextAddr);
                }
                if (useReq) req.GetResponse(); // Actually send the data
              }
            }
          } catch (Exception ex) {
            AppUI.StatusText = "Save History error: " + ex.Message;
          }
        }


        // The RequestState class passes data across async calls.
        public class RequestState
        {
            public const int BUFFER_SIZE = 100 * 1024;
            //public StringBuilder RequestData;
            public byte[] ReadBuffer;
            public WebRequest Request;
            public int RequestDataLength;
            public Stream ResponseStream;
            public FrmFind ThisFrmFind;
            // Create Decoder for appropriate enconding type.
            //public Decoder StreamDecode = Encoding.UTF8.GetDecoder();

            public RequestState(FrmFind thisFrmFind)
            {
                ReadBuffer = new byte[BUFFER_SIZE];
                Request = null;
                RequestDataLength = 0;
                ResponseStream = null;
                ThisFrmFind = thisFrmFind;
            }
        }


        /// <summary>
        /// Load the location history from disk when the form is created.
        /// Handle local disk files, http: and ftp: protocols, asynchronously.
        /// </summary>
        private void LoadHistoryFile()
        {
            if (cmbFind.AutoCompleteCustomSource != null)
            {
                //return;

                IEnumerator en = cmbFind.AutoCompleteCustomSource.GetEnumerator();
                WebRequest req = WebRequest.Create(historyFilePath);
                // Some protocol-dependent things here not handled well by default superclass:
                switch (req.GetType().Name)
                {
                    case "FtpWebRequest":
                        FtpWebRequest ftpReq = (FtpWebRequest)req;
                        ftpReq.UseBinary = false; // we're getting a text file
                        break;
                }
                
                
                if (cred != null) req.Credentials = cred; // Username/Password for remote resource if necessary
                RequestState reqState = new RequestState(this);
                // Put the request into the state object so it can be passed around.
                reqState.Request = req;
                // Issue the async request.
                IAsyncResult r = (IAsyncResult)req.BeginGetResponse(
                   new AsyncCallback(RespCallback), reqState);

                ////// Old Synchronous code
                //////WebResponse resp = null;
                //////try
                //////{
                //////    resp = req.GetResponse();
                //////}
                //////catch (Exception ex)
                //////{
                //////    return; // File might not be there, or is corrupt, or no network for remote file
                //////}
                ////////using (StreamReader sr = new StreamReader(historyFilePath))
                //////using (StreamReader sr = new StreamReader(resp.    GetResponseStream()))
                //////{
                //////    string addr;
                //////    while ((addr = sr.ReadLine()) != null)
                //////    {
                //////        cmbFind.AutoCompleteCustomSource.Add(addr);
                //////    }
                //////}
            }
        }

        private static void RespCallback(IAsyncResult asynchResult)
        {

            // Get the RequestState object from the async result.
            RequestState reqState = (RequestState)asynchResult.AsyncState;

            // Get the WebRequest from RequestState.
            WebRequest req = reqState.Request;

            // Call EndGetResponse, which produces the WebResponse object
            //  that came from the request issued above.
            WebResponse resp = null;
            try {
              resp = req.EndGetResponse(asynchResult);

            } catch (Exception ex) {
              AppUI.StatusText = ex.Message;
              return;
            }
            //  Start reading data from the response stream.
            Stream ResponseStream = resp.GetResponseStream();

            // Store the response stream in RequestState to read 
            // the stream asynchronously.
            reqState.ResponseStream = ResponseStream;

            //  Pass rs.BufferRead to BeginRead. Read data into rs.BufferRead
            IAsyncResult iarRead = ResponseStream.BeginRead(reqState.ReadBuffer, 0,
               RequestState.BUFFER_SIZE, new AsyncCallback(ReadCallBack), reqState);
        }


        private static void ReadCallBack(IAsyncResult asyncResult)
        {
            // Get the RequestState object from AsyncResult.
            RequestState reqState = (RequestState)asyncResult.AsyncState;

            // Retrieve the ResponseStream that was set in RespCallback. 
            Stream responseStream = reqState.ResponseStream;

            // Read rs.BufferRead to verify that it contains data. 
            int read = responseStream.EndRead(asyncResult);
            if (read > 0)
            {

                // Continue reading data until 
                // responseStream.EndRead returns 0.
                // Note - this logic assumes the whole file will fit in
                // BUFFER_SIZE bytes. We just append each read chunk to the buffer.
                // Will blow up badly if the remote file is bigger than BUFFER_SIZE bytes.
                reqState.RequestDataLength += read;
                IAsyncResult ar = responseStream.BeginRead(
                   reqState.ReadBuffer, read, RequestState.BUFFER_SIZE - reqState.RequestDataLength,
                   new AsyncCallback(ReadCallBack), reqState);
            }
            else
            {
                if (reqState.RequestDataLength > 0)
                {
                    //  Handle the data - fill cmb.AutoCompleteCustomSource.
                    //////string strContent;
                    //////strContent = reqState.RequestData.ToString();

                    using (StreamReader sr = new StreamReader(new MemoryStream(reqState.ReadBuffer,
                            0, reqState.RequestDataLength)))
                    {
                        String addr;
                        while (( addr = sr.ReadLine()) != null)
                        {
                            reqState.ThisFrmFind.cmbFind.AutoCompleteCustomSource.Add(addr);
                        }
                    }
                    reqState.ReadBuffer = null; // free ReadBuffer

                }
                // Close down the response stream.
                responseStream.Close();
            }
            return;
        }


    }
}